#include "main_dialog.h"
#include "ui_main_dialog.h"

#include <QDebug>
#include <QProcess>
#include <QPushButton>
#include <QRegularExpression>

#define ICM_LUX13HS_LVDS_MUTE_BL    0x8210
#define ICM_LUX13HS_LVDS_MUTE_BR    0x8214
#define ICM_LUX13HS_LVDS_MUTE_TL    0x8218
#define ICM_LUX13HS_LVDS_MUTE_TR    0x821C

//-------------------------------------
// ICM_LUX13HS_CLPOUT_CONFIG register bit definitions
//-------------------------------------
#define ICM_LUX13HS_CLPOUT_CONFIG   0x8170

// The pulse width is "+1", so specifying a 1 equates to a pulse width of 2 clocks.
#define ICMX13HS_CLPOUT_PULSE_WIDTH_MASK	(3 << 8)
#define ICMX13HS_CLPOUT_PULSE_WIDTH(x)		((x) << 8)
#define ICMX13HS_CLPOUT_DISABLE             0
#define ICMX13HS_CLPOUT_ENABLE              (1 << 12)
#define ICMX13HS_CLPOUT_PULSE2_OFFSET_MASK	0x1F


MainDialog::MainDialog(QWidget *parent) :
    QDialog(parent, Qt::WindowTitleHint | Qt::WindowSystemMenuHint | Qt::WindowCloseButtonHint),
    ui(new Ui::MainDialog)
{
    // Initial setup of the layout.
    ui->setupUi(this);

    // Initially, all masks are assumed off.
    bl_value = 0;
    br_value = 0;
    tl_value = 0;
    tr_value = 0;

    // Add all our buttons into the appropriate groups.
    bl_row.append(ui->chl_0);
    bl_row.append(ui->chl_1);
    bl_row.append(ui->chl_2);
    bl_row.append(ui->chl_3);
    bl_row.append(ui->chl_4);
    bl_row.append(ui->chl_5);
    bl_row.append(ui->chl_6);
    bl_row.append(ui->chl_7);
    bl_row.append(ui->chl_8);
    bl_row.append(ui->chl_9);
    bl_row.append(ui->chl_10);
    bl_row.append(ui->chl_11);
    bl_row.append(ui->chl_12);
    bl_row.append(ui->chl_13);
    bl_row.append(ui->chl_14);
    bl_row.append(ui->chl_15);
    bl_row.append(ui->chl_16);
    bl_row.append(ui->chl_17);
    bl_row.append(ui->chl_18);
    bl_row.append(ui->chl_19);

    br_row.append(ui->chl_100);
    br_row.append(ui->chl_101);
    br_row.append(ui->chl_102);
    br_row.append(ui->chl_103);
    br_row.append(ui->chl_104);
    br_row.append(ui->chl_105);
    br_row.append(ui->chl_106);
    br_row.append(ui->chl_107);
    br_row.append(ui->chl_108);
    br_row.append(ui->chl_109);
    br_row.append(ui->chl_110);
    br_row.append(ui->chl_111);
    br_row.append(ui->chl_112);
    br_row.append(ui->chl_113);
    br_row.append(ui->chl_114);
    br_row.append(ui->chl_115);
    br_row.append(ui->chl_116);
    br_row.append(ui->chl_117);
    br_row.append(ui->chl_118);
    br_row.append(ui->chl_119);

    tl_row.append(ui->chl_200);
    tl_row.append(ui->chl_201);
    tl_row.append(ui->chl_202);
    tl_row.append(ui->chl_203);
    tl_row.append(ui->chl_204);
    tl_row.append(ui->chl_205);
    tl_row.append(ui->chl_206);
    tl_row.append(ui->chl_207);
    tl_row.append(ui->chl_208);
    tl_row.append(ui->chl_209);
    tl_row.append(ui->chl_210);
    tl_row.append(ui->chl_211);
    tl_row.append(ui->chl_212);
    tl_row.append(ui->chl_213);
    tl_row.append(ui->chl_214);
    tl_row.append(ui->chl_215);
    tl_row.append(ui->chl_216);
    tl_row.append(ui->chl_217);
    tl_row.append(ui->chl_218);
    tl_row.append(ui->chl_219);

    tr_row.append(ui->chl_300);
    tr_row.append(ui->chl_301);
    tr_row.append(ui->chl_302);
    tr_row.append(ui->chl_303);
    tr_row.append(ui->chl_304);
    tr_row.append(ui->chl_305);
    tr_row.append(ui->chl_306);
    tr_row.append(ui->chl_307);
    tr_row.append(ui->chl_308);
    tr_row.append(ui->chl_309);
    tr_row.append(ui->chl_310);
    tr_row.append(ui->chl_311);
    tr_row.append(ui->chl_312);
    tr_row.append(ui->chl_313);
    tr_row.append(ui->chl_314);
    tr_row.append(ui->chl_315);
    tr_row.append(ui->chl_316);
    tr_row.append(ui->chl_317);
    tr_row.append(ui->chl_318);
    tr_row.append(ui->chl_319);


    // Setup our row buttons.
    connect(ui->pb_bl, SIGNAL(clicked()), this, SLOT(check_bl()));
    connect(ui->pb_br, SIGNAL(clicked()), this, SLOT(check_br()));
    connect(ui->pb_tl, SIGNAL(clicked()), this, SLOT(check_tl()));
    connect(ui->pb_tr, SIGNAL(clicked()), this, SLOT(check_tr()));

    // Setup our pushbutton connections.
    connect(ui->chl_0, SIGNAL(clicked()), this, SLOT(chl_select()));
    connect(ui->chl_1, SIGNAL(clicked()), this, SLOT(chl_select()));
    connect(ui->chl_2, SIGNAL(clicked()), this, SLOT(chl_select()));
    connect(ui->chl_3, SIGNAL(clicked()), this, SLOT(chl_select()));
    connect(ui->chl_4, SIGNAL(clicked()), this, SLOT(chl_select()));
    connect(ui->chl_5, SIGNAL(clicked()), this, SLOT(chl_select()));
    connect(ui->chl_6, SIGNAL(clicked()), this, SLOT(chl_select()));
    connect(ui->chl_7, SIGNAL(clicked()), this, SLOT(chl_select()));
    connect(ui->chl_8, SIGNAL(clicked()), this, SLOT(chl_select()));
    connect(ui->chl_9, SIGNAL(clicked()), this, SLOT(chl_select()));
    connect(ui->chl_10, SIGNAL(clicked()), this, SLOT(chl_select()));
    connect(ui->chl_11, SIGNAL(clicked()), this, SLOT(chl_select()));
    connect(ui->chl_12, SIGNAL(clicked()), this, SLOT(chl_select()));
    connect(ui->chl_13, SIGNAL(clicked()), this, SLOT(chl_select()));
    connect(ui->chl_14, SIGNAL(clicked()), this, SLOT(chl_select()));
    connect(ui->chl_15, SIGNAL(clicked()), this, SLOT(chl_select()));
    connect(ui->chl_16, SIGNAL(clicked()), this, SLOT(chl_select()));
    connect(ui->chl_17, SIGNAL(clicked()), this, SLOT(chl_select()));
    connect(ui->chl_18, SIGNAL(clicked()), this, SLOT(chl_select()));
    connect(ui->chl_19, SIGNAL(clicked()), this, SLOT(chl_select()));

    connect(ui->chl_100, SIGNAL(clicked()), this, SLOT(chl_select()));
    connect(ui->chl_101, SIGNAL(clicked()), this, SLOT(chl_select()));
    connect(ui->chl_102, SIGNAL(clicked()), this, SLOT(chl_select()));
    connect(ui->chl_103, SIGNAL(clicked()), this, SLOT(chl_select()));
    connect(ui->chl_104, SIGNAL(clicked()), this, SLOT(chl_select()));
    connect(ui->chl_105, SIGNAL(clicked()), this, SLOT(chl_select()));
    connect(ui->chl_106, SIGNAL(clicked()), this, SLOT(chl_select()));
    connect(ui->chl_107, SIGNAL(clicked()), this, SLOT(chl_select()));
    connect(ui->chl_108, SIGNAL(clicked()), this, SLOT(chl_select()));
    connect(ui->chl_109, SIGNAL(clicked()), this, SLOT(chl_select()));
    connect(ui->chl_110, SIGNAL(clicked()), this, SLOT(chl_select()));
    connect(ui->chl_111, SIGNAL(clicked()), this, SLOT(chl_select()));
    connect(ui->chl_112, SIGNAL(clicked()), this, SLOT(chl_select()));
    connect(ui->chl_113, SIGNAL(clicked()), this, SLOT(chl_select()));
    connect(ui->chl_114, SIGNAL(clicked()), this, SLOT(chl_select()));
    connect(ui->chl_115, SIGNAL(clicked()), this, SLOT(chl_select()));
    connect(ui->chl_116, SIGNAL(clicked()), this, SLOT(chl_select()));
    connect(ui->chl_117, SIGNAL(clicked()), this, SLOT(chl_select()));
    connect(ui->chl_118, SIGNAL(clicked()), this, SLOT(chl_select()));
    connect(ui->chl_119, SIGNAL(clicked()), this, SLOT(chl_select()));

    connect(ui->chl_200, SIGNAL(clicked()), this, SLOT(chl_select()));
    connect(ui->chl_201, SIGNAL(clicked()), this, SLOT(chl_select()));
    connect(ui->chl_202, SIGNAL(clicked()), this, SLOT(chl_select()));
    connect(ui->chl_203, SIGNAL(clicked()), this, SLOT(chl_select()));
    connect(ui->chl_204, SIGNAL(clicked()), this, SLOT(chl_select()));
    connect(ui->chl_205, SIGNAL(clicked()), this, SLOT(chl_select()));
    connect(ui->chl_206, SIGNAL(clicked()), this, SLOT(chl_select()));
    connect(ui->chl_207, SIGNAL(clicked()), this, SLOT(chl_select()));
    connect(ui->chl_208, SIGNAL(clicked()), this, SLOT(chl_select()));
    connect(ui->chl_209, SIGNAL(clicked()), this, SLOT(chl_select()));
    connect(ui->chl_210, SIGNAL(clicked()), this, SLOT(chl_select()));
    connect(ui->chl_211, SIGNAL(clicked()), this, SLOT(chl_select()));
    connect(ui->chl_212, SIGNAL(clicked()), this, SLOT(chl_select()));
    connect(ui->chl_213, SIGNAL(clicked()), this, SLOT(chl_select()));
    connect(ui->chl_214, SIGNAL(clicked()), this, SLOT(chl_select()));
    connect(ui->chl_215, SIGNAL(clicked()), this, SLOT(chl_select()));
    connect(ui->chl_216, SIGNAL(clicked()), this, SLOT(chl_select()));
    connect(ui->chl_217, SIGNAL(clicked()), this, SLOT(chl_select()));
    connect(ui->chl_218, SIGNAL(clicked()), this, SLOT(chl_select()));
    connect(ui->chl_219, SIGNAL(clicked()), this, SLOT(chl_select()));

    connect(ui->chl_300, SIGNAL(clicked()), this, SLOT(chl_select()));
    connect(ui->chl_301, SIGNAL(clicked()), this, SLOT(chl_select()));
    connect(ui->chl_302, SIGNAL(clicked()), this, SLOT(chl_select()));
    connect(ui->chl_303, SIGNAL(clicked()), this, SLOT(chl_select()));
    connect(ui->chl_304, SIGNAL(clicked()), this, SLOT(chl_select()));
    connect(ui->chl_305, SIGNAL(clicked()), this, SLOT(chl_select()));
    connect(ui->chl_306, SIGNAL(clicked()), this, SLOT(chl_select()));
    connect(ui->chl_307, SIGNAL(clicked()), this, SLOT(chl_select()));
    connect(ui->chl_308, SIGNAL(clicked()), this, SLOT(chl_select()));
    connect(ui->chl_309, SIGNAL(clicked()), this, SLOT(chl_select()));
    connect(ui->chl_310, SIGNAL(clicked()), this, SLOT(chl_select()));
    connect(ui->chl_311, SIGNAL(clicked()), this, SLOT(chl_select()));
    connect(ui->chl_312, SIGNAL(clicked()), this, SLOT(chl_select()));
    connect(ui->chl_313, SIGNAL(clicked()), this, SLOT(chl_select()));
    connect(ui->chl_314, SIGNAL(clicked()), this, SLOT(chl_select()));
    connect(ui->chl_315, SIGNAL(clicked()), this, SLOT(chl_select()));
    connect(ui->chl_316, SIGNAL(clicked()), this, SLOT(chl_select()));
    connect(ui->chl_317, SIGNAL(clicked()), this, SLOT(chl_select()));
    connect(ui->chl_318, SIGNAL(clicked()), this, SLOT(chl_select()));
    connect(ui->chl_319, SIGNAL(clicked()), this, SLOT(chl_select()));

    connect(ui->cb_clpout, SIGNAL(clicked()), this, SLOT(check_clpout()));
    connect(ui->sb_clpout_offset, SIGNAL(valueChanged(int)), this, SLOT(clpout_offset_change(int)));
    connect(ui->sb_clpout_width, SIGNAL(valueChanged(int)), this, SLOT(clpout_width_change(int)));
}

MainDialog::~MainDialog()
{
    delete ui;
}

void MainDialog::chl_select()
{
    uint32_t reg_addr;
    uint32_t reg_value;

    // Get the number out of the name.
    QRegularExpression re("\\d+");
    QPushButton *pb = qobject_cast<QPushButton *>(sender());
    QRegularExpressionMatch match = re.match(pb->objectName());

    // We don't have to be too careful here, 'cuz we created the pushbuttons and we
    // know what the names look like.
    int pb_id = match.capturedTexts().first().toInt();

    // Setup the address and mask.
    if (pb_id < 100)
    {
        // Bottom left.
        reg_addr = ICM_LUX13HS_LVDS_MUTE_BL;

        // Are we setting or clearing?
        if (pb->isChecked())
            bl_value |= (1 << pb_id);
        else
            bl_value &= ~(1 << pb_id);
        reg_value = bl_value;
    }
    else if (pb_id < 200)
    {
        // Bottom right.
        reg_addr = ICM_LUX13HS_LVDS_MUTE_BR;

        // Are we setting or clearing?
        if (pb->isChecked())
            br_value |= (1 << (pb_id - 100));
        else
            br_value &= ~(1 << (pb_id - 100));
        reg_value = br_value;
    }
    else if (pb_id < 300)
    {
        // Top left.
        reg_addr = ICM_LUX13HS_LVDS_MUTE_TL;

        // Are we setting or clearing?
        if (pb->isChecked())
            tl_value |= (1 << (pb_id - 200));
        else
            tl_value &= ~(1 << (pb_id - 200));
        reg_value = tl_value;
    }
    else
    {
        // Top right.
        reg_addr = ICM_LUX13HS_LVDS_MUTE_TR;

        // Are we setting or clearing?
        if (pb->isChecked())
            tr_value |= (1 << (pb_id - 300));
        else
            tr_value &= ~(1 << (pb_id - 300));
        reg_value = tr_value;
    }

    QString command = QString("echo w 0x%1 0x%2 > /proc/fi_pci/regs").arg(reg_addr, 0, 16).arg(reg_value, 0, 16);
    system(qPrintable(command));
}

void MainDialog::check_bl()
{
    if (ui->pb_bl->isChecked())
        bl_value = 0xFFFFF;
    else
        bl_value = 0;

    QString command = QString("echo w 0x%1 0x%2 > /proc/fi_pci/regs").arg(ICM_LUX13HS_LVDS_MUTE_BL, 0, 16).arg(bl_value, 0, 16);
    system(qPrintable(command));

    for (int i = 0; i < bl_row.count(); i++)
        bl_row.at(i)->setChecked(((bl_value & (1 << i)) == ((uint32_t)1 << i)));
}

void MainDialog::check_br()
{
    if (ui->pb_br->isChecked())
        br_value = 0xFFFFF;
    else
        br_value = 0;

    QString command = QString("echo w 0x%1 0x%2 > /proc/fi_pci/regs").arg(ICM_LUX13HS_LVDS_MUTE_BR, 0, 16).arg(br_value, 0, 16);
    system(qPrintable(command));

    for (int i = 0; i < br_row.count(); i++)
        br_row.at(i)->setChecked(((br_value & (1 << i)) == ((uint32_t)1 << i)));
}

void MainDialog::check_tl()
{
    if (ui->pb_tl->isChecked())
        tl_value = 0xFFFFF;
    else
        tl_value = 0;

    QString command = QString("echo w 0x%1 0x%2 > /proc/fi_pci/regs").arg(ICM_LUX13HS_LVDS_MUTE_TL, 0, 16).arg(tl_value, 0, 16);
    system(qPrintable(command));

    for (int i = 0; i < tl_row.count(); i++)
        tl_row.at(i)->setChecked(((tl_value & (1 << i)) == ((uint32_t)1 << i)));
}

void MainDialog::check_tr()
{
    if (ui->pb_tr->isChecked())
        tr_value = 0xFFFFF;
    else
        tr_value = 0;

    QString command = QString("echo w 0x%1 0x%2 > /proc/fi_pci/regs").arg(ICM_LUX13HS_LVDS_MUTE_TR, 0, 16).arg(tr_value, 0, 16);
    system(qPrintable(command));

    for (int i = 0; i < tr_row.count(); i++)
        tr_row.at(i)->setChecked(((tr_value & (1 << i)) == ((uint32_t)1 << i)));
}

void MainDialog::check_clpout()
{
    uint32_t reg_value;

    if (ui->cb_clpout->isChecked())
    {
        reg_value = ICMX13HS_CLPOUT_ENABLE | ICMX13HS_CLPOUT_PULSE_WIDTH(ui->sb_clpout_width->value()) | (ICMX13HS_CLPOUT_PULSE2_OFFSET_MASK & ui->sb_clpout_offset->value());
    }
    else
    {
        reg_value = ICMX13HS_CLPOUT_DISABLE | ICMX13HS_CLPOUT_PULSE_WIDTH(ui->sb_clpout_width->value()) | (ICMX13HS_CLPOUT_PULSE2_OFFSET_MASK & ui->sb_clpout_offset->value());;
    }

    QString command = QString("echo w 0x%1 0x%2 > /proc/fi_pci/regs").arg(ICM_LUX13HS_CLPOUT_CONFIG, 0, 16).arg(reg_value, 0, 16);
    system(qPrintable(command));
}

void MainDialog::clpout_offset_change(int new_value)
{
    Q_UNUSED(new_value)

    check_clpout();
}

void MainDialog::clpout_width_change(int new_value)
{
    Q_UNUSED(new_value)

    check_clpout();
}
