/*!
 * @file fi_log.c
 * @brief Generic logging functions for debug or production code.
 *
 * Copyright 2015 (c) Fastec Imaging as an unpublished work.
 * All Rights Reserved.
 *
 * The information contained herein is confidential property of
 * Fastec Imaging. The use, copying, transfer or disclosure of such
 * information is prohibited except by express written agreement
 * with Fastec Imaging.
 */

//--------------------------------------------------------------------
// Includes
//--------------------------------------------------------------------

#include <stdint.h>
#include <stdarg.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <syslog.h>

#include "fi_log.h"

//--------------------------------------------------------------------
// Constant, Typedefs and Macros
//--------------------------------------------------------------------

//--------------------------------------------------------------------
// Variable declarations
//--------------------------------------------------------------------

// PDEBUG verbosity flags.
unsigned verbosity = 0;

// Not using this yet, but in our bag for when we switch to syslog.
static int debug_to_log = 0;

// Not using this yet, but in our bag for when we start using syslog.
static int use_syslog = 0;

//============================================================================
// Implementation
//============================================================================

/*
 * @brief Output a message logged via PDEBUG.
 */
void debug_log( const char *fmt, ... )
{
    va_list args;
    va_start( args, fmt );

    if ( debug_to_log )
	vsyslog( LOG_DEBUG, fmt, args );
    else
    {
	vprintf( fmt, args );
	printf( "\n" );
    }

    va_end( args );
}

/*
 * @brief Output a message logged vi fi_log().
 */
void fi_log( unsigned severity, const char *fmt, ... )
{
    va_list args;
    va_start( args, fmt );

    if ( use_syslog )
	vsyslog( severity, fmt, args );
    else
    {
	vprintf( fmt, args );
	printf( "\n" );
    }

    va_end( args );
}

/*
 * @brief Output a message for a panic-y situation.
 */
void fi_error_log_unlikely( const char *func, int line, int errno_value )
{
    fprintf( stderr, "ERROR#%d at %s#%d: %s\n", errno_value, func, line,
	     strerror(errno_value) );
}

